# ITHIM Functions for ITHIM TOOL
# June 2019 - Tomari/Maizlish
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ////////////////////////////////////////////////////////////////////////////

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Define method for is.nan for data.frame
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# https://stackoverflow.com/questions/18142117/how-to-replace-nan-value-with-
# zero-in-a-huge-data-frame. See more on S3 methods here:
# https://adv-r.hadley.nz/s3.html#s3-methods
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' Adds functionality (specifically an S3 method) to is.nan() to account for 
#' data.frames.
is.nan.data.frame <- function(x) {
  do.call(cbind, lapply(x, is.nan))
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Setup Functions ----
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# FUNCTION: Age Labels
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' Converts default age ranges
#' 
#' @param x integer or character
#' @return character
fn_age_groups <- function(x) { 
  if    (is.na(x)) y <- "NA"
  else if (x==0)   y <- "00-04"
  else if (x==5)   y <- "05-14"
  else if (x==15)  y <- "15-29"
  else if (x==30)  y <- "30-44"
  else if (x==45)  y <- "45-59"
  else if (x==60)  y <- "60-69"
  else if (x==70)  y <- "70-79"
  else if (x==80)  y <- "80+"
  else y <- "Total"
  return(y)
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# FUNCTION: Prepare the "run"
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' Modifies the input variables from the run() reactive function and adds
#' default variables loaded in preprocessing to match the settings the rest of
#' the Analytic Engine expects. 
#' 
#' @param l is a list with baseline, scenario, fRegion, fGeography, and fYear.
#' @return a list with no CHANGES to above inputs, and these ADDITIONS:
#' Walkspeed, Bikespeed, SiN, PAChronicBeta,
#' PAAllCauseBeta, NQtiles, VSL, bdistance,
#' sdistance, at, facility, bus_occupancy,
#' age_sex_pop, wbt, cv, nontravelMETS,
#' PM25_VMT, rti, age_sex_region_year, cbd,
#' riskadjust, co2, regions, infographic,
#' prefab_infographic, METminWalk_Bike, pa_rr, apc
fn_prep <- function(l){
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  # Note, this function, alongside
  # fn_distances, are perhaps the most
  # confusing for those comparing the
  # Long Version of the ITHIM Analytic
  # Engine with the modularized version
  # of the Shiny app. For those looking
  # at this code having learned the Long
  # Version, you should note that
  # fn_prep and fn_distances is a
  # mixture of the Long Version's
  # scenario_distance.function and the
  # "Main Body of Program" section.
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  # PARAMETERS ----
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  l$Walkspeed <- d$parameters[d$parameters$VariableName == "Walkspeed", 3]
  l$Bikespeed <- d$parameters[d$parameters$VariableName == "Bikespeed", 3]
  
  # Safety in Numbers Coefficient (SiN)
  l$SiN <- d$parameters[d$parameters$VariableName == "SiN", 3]
  
  # Physical Activity CRF coeeficients for Chronic Diseases and All Causes
  l$PAChronicBeta <-
    d$parameters[d$parameters$VariableName == "PAChronicBeta", 3]
  l$PAAllCauseBeta <-
    d$parameters[d$parameters$VariableName == "PAAllCauseBeta", 3]
  
  # Number of quantiles for physical activity distribution
  l$NQtiles <- d$parameters[d$parameters$VariableName == "NQtiles", 3]
  
  # Value of a statistical life
  l$VSL <- d$parameters[d$parameters$VariableName == "VSL", 3]
  
# Users can select the percentage of car electrification
# l$LCD_VMT <-
#    d$parameters[d$parameters$VariableName == "LCD_VMT", c("Default")]
#     
  
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  # DISTANCES & REGIONS ----
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  # Note, baseline (and some scenario)
  # DISTANCES have already been provided
  # to us from the shiny reactive
  # function `run`.
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  # SUBSET DISTANCES BY REGION
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  # BASELINE
  l$bdistance <- l$baseline$Distances[
    which(l$baseline$Distances$Region==l$fRegion), 
    c("Region", "Mode", "Item_Result")]
  names(l$bdistance)[3] <- "Baseline"

  # SCENARIOS
  # Assuming there is a non-Absolute-Time/Multiples scenario.
  # In other words, SCS2040, etc... and User Upload.
  if(!is.null(l$scenario$Distances)){
    l$sdistance <- l$scenario$Distances[
      which(l$scenario$Distances$Region==l$fRegion),
      c("Region", "Mode", "Item_Result")]
    names(l$sdistance)[3] <- "Scenario"
  } else {
    # if it is absolute/fixed time OR multiples OR LCD, copy baseline distances.
    l$sdistance <- l$baseline$Distances[
      which(l$baseline$Distances$Region==l$fRegion),
      c("Region", "Mode", "Item_Result")]
    names(l$sdistance)[3] <- "Scenario"
  }
  
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  # ADD & SUBSET data files by fRegion
  condition <- sapply(d, function(x) if("Region" %in% names(x)) T else F)
  # grab data, but only ones with "Region" columns.
  rlist <- d[condition]
  # apply subset by region
  rlist2 <- lapply(rlist, function(x) x[which(x$Region==l$fRegion),])
  l <- append(l, rlist2)  # add to l
  
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  # SUBSET AGE_SEX_REGION_YEAR by Geography/Year
  #Change 2010 to 2015
  age_sex_region_year2010 <- l$age_sex_region_year[
    which(l$age_sex_region_year$Geography == l$fGeography &
            l$age_sex_region_year$Year == 2015),]
  
  l$age_sex_region_year <- l$age_sex_region_year[
    which(l$age_sex_region_year$Geography == l$fGeography &
            l$age_sex_region_year$Year == l$fYear),]
  
  # SUBSET co2 by Year
  l$co2 <- l$co2[which(l$co2$Year == l$fYear),]
  
  # SUBSET PM25_VMT by year
  l$PM25_VMT <- l$PM25_VMT[which(l$PM25_VMT$Year == l$fYear),]
  
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  # population adjustment factor for RTIs based on SWITRS data
  l$pop_adjust <- 
    sum(l$age_sex_region_year$Population, na.rm=TRUE)/
    sum(age_sex_region_year2010$Population, na.rm=TRUE)
  
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  # SIMPLIFY bus occupancy
  l$bus_occupancy <- l$bus_occupancy[1,"Occupancy"]
  
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  # ADD & FIX AGE data.frames
  # Add data.frames with Age from d to l
  condition <- sapply(d, function(x) if("Age" %in% colnames(x)) T else F)
  alist <- d[condition]  # subset d with data.frames with Age.
  alist2 <- alist[!(names(alist) %in% names(l))]  # get dfs with ages
  l <- append(l, alist2)  # add new ages data.frames.
  
  # Fix ages in l
  condition <- sapply(l, function(x) if("Age" %in% colnames(x)) T else F)
  alist3 <- l[condition]
  for(i in seq_along(alist3)){
    alist3[[i]]$Age <- sapply(alist3[[i]]$Age, fn_age_groups)
  }
  l[condition] <- alist3  # replace fixed data.frames
  
  l  # return
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Core ITHIM Functions ----
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# FUNCTION: Distances
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#'
#'
#' @param l is a list of values from fn_prep() (see above).
#' @return a list with no CHANGES to above inputs, and these ADDITIONS:
#' at_wide, bmiles_wide, smiles_wide, smiles_long, bs_distance_mode
fn_distances <- function(l){

  
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  # Note, this function, alongside
  # fn_prep, are perhaps the most
  # confusing for those comparing the
  # Long Version of the ITHIM Analytic
  # Engine with the modularized version
  # of the Shiny app. For those looking
  # at this code having learned the Long
  # Version, you should note that
  # fn_prep and fn_distances is a
  # mixture of the Long Version's
  # scenario_distance.function and the
  # "Main Body of Program" section.
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  # If User Uploaded
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  # Convert from daily to annual distances
  if(attr(l$scenario, "scenario_type") == "user_defined"){
    l$bdistance$Baseline <- l$bdistance$Baseline * 365
    l$sdistance$Scenario <- l$sdistance$Scenario * 365
  }
  
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  # BASELINE ----
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  # (See section: 1. Exposures: Travel Times and Distances in Long Version.)
  # Baseline AT Times and Distances
  # Read age-sex specific walking and cycling MET weights

  # Reshape baseline and pre-rendered scenario AT files from long to wide
  l$at_wide <-
    reshape(
      l$at,
      timevar = "Mode",
      idvar = c("Region", "Age", "Sex"),
      direction = "wide"
    )
  
  # Create total from walk and bike mean times
  l$at_wide$Baseline.Total <- l$at_wide$Baseline.Walk + 
    l$at_wide$Baseline.Bike
  
  # Reshape baseline distances by mode
  l$bmiles_wide <-
    reshape(
      l$bdistance,
      timevar = "Mode",
      idvar = c("Region"),
      direction = "wide"
    )
  
  # Amount of car miles substitution for walk and bike
  p <- l$p  
  
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  # SCENARIO DISTANCES ----
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  # (See section: Calculate Travel Distances by Mode for Baseline and Scenario
  # in Long Version.)
  
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  # If User Uploaded
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  if(attr(l$scenario, "scenario_type") == "user_defined"){
    
    # Select proportion of miles by roadway type for baseline
    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    # Note, that since we've split
    # up the Long Version code, we're
    # going straight to creating
    # `bfacility`. Whereas, in the Long
    # Version, this occurs directly after
    # `sdistance` has been created.
    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    l$bfacility <- 
      l$baseline$`Proportion of Vehicle Miles by Mode and Facility Type`
    
    l$bfacility <- l$bfacility[which(l$bfacility$Region==l$fRegion),
                               c("Region", 
                                 "Mode", 
                                 "Strata", 
                                 "Item_Result")]
    
    # create proportion by roadway type wide file
    l$bfacility_wide <- reshape(l$bfacility, 
                              timevar = "Strata", 
                              idvar = c("Region","Mode"), 
                              direction = "wide")
    
    names(l$bfacility_wide)[3:5] <- c("art_pct_b", 
                                      "hwy_pct_b", 
                                      "local_pct_b")
    
    # Align columns for rbind
    l$bfacility_wide <- l$bfacility_wide[,c("Region",
                                            "Mode", 
                                            "local_pct_b", 
                                            "art_pct_b", 
                                            "hwy_pct_b")]
    
    l$bfacility_wide <- 
      rbind(l$facility[l$facility$Mode %in% c("Walk","Bike"),],
            l$bfacility_wide)
    
    # add motorcycle using car distribution
    l$bfacility_wide <- 
      rbind(l$bfacility_wide,
            l$bfacility_wide[l$bfacility_wide$Mode=="Car",])
    
    # rename motorcycle mode
    l$bfacility_wide[6,2] <- "Motorcycle"
    
    # Select proportion of miles by roadway type for scenario
    l$sfacility <- 
      l$scenario$`Proportion of Vehicle Miles by Mode and Facility Type`
    
    l$sfacility <- l$sfacility[which(l$sfacility$Region==l$fRegion),
                               c("Region", 
                                 "Mode", 
                                 "Strata", 
                                 "Item_Result")]
    
    # create proportion by roadway type wide file
    l$sfacility_wide <- reshape(l$sfacility, 
                              timevar = "Strata", 
                              idvar = c("Region","Mode"), 
                              direction = "wide")
    
    names(l$sfacility_wide)[3:5] <- c("art_pct_b", 
                                      "hwy_pct_b", 
                                      "local_pct_b")
    
    # Align columns for rbind
    l$sfacility_wide <- l$sfacility_wide[,c("Region", 
                                            "Mode", 
                                            "local_pct_b", 
                                            "art_pct_b", 
                                            "hwy_pct_b")]
    
    l$sfacility_wide <- 
      rbind(l$facility[l$facility$Mode %in% c("Walk","Bike"),],
            l$sfacility_wide)
    
    # add motorcycle using car distribution
    l$sfacility_wide <- 
      rbind(l$sfacility_wide,
            l$sfacility_wide[l$sfacility_wide$Mode=="Car",])
    
    # rename motorcycle mode
    l$sfacility_wide[6,2] <- "Motorcycle"
    
    # rename scenario proportion columns
    names(l$sfacility_wide)[3:5] <- c("local_pct_s", 
                                      "art_pct_s", 
                                      "hwy_pct_s")
  }
  
  # Reshape Distance file from long to wide
  l$smiles_wide <- reshape(
    l$sdistance,
    timevar = "Mode",
    idvar = c("Region"),
    direction = "wide"
  )
  
  
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  # If Multiples
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  if(attr(l$scenario, "scenario_type") == "multiples"){
    walk_multiple <- l$scenario$multiples[1]
    bike_multiple <- l$scenario$multiples[2]
    transit_multiple <- l$scenario$multiples[3]
    
    l$smiles_wide$Scenario.Walk <-
      l$bmiles_wide$Baseline.Walk * walk_multiple
    l$smiles_wide$Scenario.Bike <-
      l$bmiles_wide$Baseline.Bike * bike_multiple
    l$smiles_wide$Scenario.Bus  <-
      l$bmiles_wide$Baseline.Bus  * transit_multiple
    l$smiles_wide$Scenario.Rail <-
      l$bmiles_wide$Baseline.Rail * transit_multiple
    
    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    # Reduce Car miles by the increase in
    # walk and bike miles taking into
    # account proportion of substitution
    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    l$smiles_wide$Scenario.CarDriver <-
      (1 - p) * l$bmiles_wide$Baseline.CarDriver + p *    ((
        l$bmiles_wide$Baseline.Walk + 
          l$bmiles_wide$Baseline.Bike + 
          l$bmiles_wide$Baseline.Rail + 
          l$bmiles_wide$Baseline.Bus + 
          l$bmiles_wide$Baseline.CarDriver + 
          l$bmiles_wide$Baseline.CarPassenger
      ) - (
        l$smiles_wide$Scenario.Walk + 
          l$smiles_wide$Scenario.Bike + 
          l$smiles_wide$Scenario.Rail + 
          l$smiles_wide$Scenario.Bus
      )
      ) * (
        l$bmiles_wide$Baseline.CarDriver / (
          l$bmiles_wide$Baseline.CarDriver + 
            l$bmiles_wide$Baseline.CarPassenger
        )
      )
    l$smiles_wide$Scenario.CarPassenger <-
      (1 - p) * l$bmiles_wide$Baseline.CarPassenger + p * ((
        l$bmiles_wide$Baseline.Walk + 
          l$bmiles_wide$Baseline.Bike + 
          l$bmiles_wide$Baseline.Rail + 
          l$bmiles_wide$Baseline.Bus + 
          l$bmiles_wide$Baseline.CarDriver + 
          l$bmiles_wide$Baseline.CarPassenger
      ) - (
        l$smiles_wide$Scenario.Walk + 
          l$smiles_wide$Scenario.Bike + 
          l$smiles_wide$Scenario.Rail + 
          l$smiles_wide$Scenario.Bus
      )
      ) * (
        l$bmiles_wide$Baseline.CarPassenger / (
          l$bmiles_wide$Baseline.CarDriver + 
            l$bmiles_wide$Baseline.CarPassenger
        )
      )
    
  }
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  # If Absolute Time
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  if(attr(l$scenario, "scenario_type") == "absolute_time"){
    Scenario.Walk <- l$scenario$absolute_time[1]
    Scenario.Bike <- l$scenario$absolute_time[2]
    l$smiles_wide$Scenario.Walk <- Scenario.Walk*(l$Walkspeed/60)*52
    l$smiles_wide$Scenario.Bike <- Scenario.Bike*(l$Bikespeed/60)*52
    
    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    # Reduce Car miles by the increase in
    # walk and bike miles taking into
    # account proportion of substitution
    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    l$smiles_wide$Scenario.CarDriver <-
      (1 - p) * l$bmiles_wide$Baseline.CarDriver + p * ((
        l$bmiles_wide$Baseline.Walk + 
          l$bmiles_wide$Baseline.Bike + 
          l$bmiles_wide$Baseline.CarDriver + 
          l$bmiles_wide$Baseline.CarPassenger
      ) - (l$smiles_wide$Scenario.Walk + 
             l$smiles_wide$Scenario.Bike)
      ) * (
        l$bmiles_wide$Baseline.CarDriver / (
          l$bmiles_wide$Baseline.CarDriver + 
            l$bmiles_wide$Baseline.CarPassenger
        )
      )
    l$smiles_wide$Scenario.CarPassenger <-
      (1 - p) * l$bmiles_wide$Baseline.CarPassenger + p * ((
        l$bmiles_wide$Baseline.Walk + 
          l$bmiles_wide$Baseline.Bike + 
          l$bmiles_wide$Baseline.CarDriver + 
          l$bmiles_wide$Baseline.CarPassenger
      ) - (l$smiles_wide$Scenario.Walk + l$smiles_wide$Scenario.Bike)
      ) * (
        l$bmiles_wide$Baseline.CarPassenger / (
          l$bmiles_wide$Baseline.CarDriver + 
            l$bmiles_wide$Baseline.CarPassenger
        )
      )
  }

  # Create Table for interface presentation
  tmp <- names(l$smiles_wide)[2:length(l$smiles_wide)]
  tmp2 <- sub("Scenario\\.", "", tmp)
  
  l$smiles_long <-
    reshape(
      l$smiles_wide,
      varying = tmp,
      v.names = c("Scenario"),
      timevar = "Mode",
      times = tmp2,
      new.row.names = 1:8,
      direction = "long"
    )
  
  l$smiles_long <- l$smiles_long[order(l$smiles_long$Mode) ,]
  l$bdistance <- l$bdistance[order(l$bdistance$Mode) ,]
  l$bs_distance_mode <-
    cbind(l$bdistance, l$smiles_long["Scenario"])
  
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  l # return
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# FUNCTION: Scenario Times 
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' Obtain Scenario Times
#' 
#' @param l is a list of values from fn_distances() (see above).
#' @return a list with no CHANGES to above inputs, and these ADDITIONS:
#' sat_wide, at_sat_wide, transit_AT_long
fn_scenario_time <- function(l){
  # browser()
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  # Convert Mean Walk and Bike Distances
  # of chosen scenario to times based on
  # speed
  Scenario.Walk <- l$smiles_wide$Scenario.Walk * 60 / (52 * l$Walkspeed)
  Scenario.Bike <- l$smiles_wide$Scenario.Bike * 60 / (52 * l$Bikespeed)
  
  # Calculate mean active travel time
  # for User Upload (when default
  # calibration baseline not used)
  Baseline.Walk <- l$bmiles_wide$Baseline.Walk * 60 / (52 * l$Walkspeed)
  Baseline.Bike <- l$bmiles_wide$Baseline.Bike * 60 / (52 * l$Bikespeed) 
  
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  # Distribute Mean Times by Baseline age-sex ratios
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  
  # Calculate baseline age-sex walk bike AT time ratios
  at_ratios <- l$at_wide[2:17, ]
  at_ratios$Baseline.Walk <-
    at_ratios$Baseline.Walk / 
    at_ratios[at_ratios$Sex == "2" &
                at_ratios$Age == "15-29", 
              "Baseline.Walk"]
  
  at_ratios$Baseline.Bike <-
    at_ratios$Baseline.Bike / 
    at_ratios[at_ratios$Sex == "2" &
                at_ratios$Age == "15-29", 
              "Baseline.Bike"]
  
  # Append population Percent to ratios file
  at_ratios$Percent <- l$age_sex_pop[2:17, "Percent"]
  
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  # Calculate the Scenario Walk and Bike
  # mean AT minutes by age and sex from
  # overall mean and age-sex ratios
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  # Walk times
  walk_ratio_sum <- at_ratios$Baseline.Walk * at_ratios$Percent
  walk_ratio_sum <- sum(walk_ratio_sum[1:16])
  
  # This calculates Scenario walk mean minutes by age and sex
  at_means <- at_ratios
  at_means$Scenario.Walk <- 
    (at_ratios$Baseline.Walk * 
       at_ratios$Percent / 
       walk_ratio_sum) * 
    Scenario.Walk / 
    at_ratios$Percent
  
  # Bike times
  bike_ratio_sum <- at_ratios$Baseline.Bike * at_ratios$Percent
  bike_ratio_sum <- sum(bike_ratio_sum[1:16])
  
  # This caculates Scenario bike mean minutes by age and sex
  at_means$Scenario.Bike <- 
    (at_ratios$Baseline.Bike * 
       at_ratios$Percent / 
       bike_ratio_sum) * 
    Scenario.Bike / 
    at_ratios$Percent
  
  # Total Time
  at_means$Scenario.Total <- 
    at_means$Scenario.Walk + 
    at_means$Scenario.Bike
  
  #Save columns
  sat_wide <- at_means[, c(1:3, 8:10)]
  
  # Create Scenario AT Times file
  # Add total line
  sat <- l$at_wide[1, 1:3]
  
  # Code correct for rounding error for Multiples - begin  
  if(attr(l$scenario, "scenario_type") == "multiples"){
    
    # if walk multiple is 1, change Scenario.Walk in sat_wide/sat
    if(l$scenario$multiples[1] == 1){
      sat_wide$Scenario.Walk <- l$at_wide[2:17,c("Baseline.Walk")]
      Scenario.Walk <- l$at_wide[1,c("Baseline.Walk")]
    }
    
    # if bike multiple is 1, change Scenario.Bike in sat_wide/sat
    if(l$scenario$multiples[2] == 1){
      sat_wide$Scenario.Bike <- l$at_wide[2:17,c("Baseline.Bike")]
      Scenario.Bike <- l$at_wide[1,c("Baseline.Bike")]
    }
    
    # if either bike/walk multiple is 1 change Scenario.Total
    # Note: this is not redundant with previous two if statements.
    if(l$scenario$multiples[1] == 1 | l$scenario$multiples[2] == 1){
      sat_wide$Scenario.Total <- 
        sat_wide$Scenario.Walk + 
        sat_wide$Scenario.Bike
    }
  }
  
  # Code to Correct for LCD
  if(l$scenario$name == "Low Carbon Driving"){

    sat_wide$Scenario.Walk <- l$at_wide[2:17,c("Baseline.Walk")]
    sat_wide$Scenario.Bike <- l$at_wide[2:17,c("Baseline.Bike")]
    sat_wide$Scenario.Total <- l$at_wide[2:17,c("Baseline.Total")]
    Scenario.Walk <- l$at_wide[1,c("Baseline.Walk")]  
    Scenario.Bike <- l$at_wide[1,c("Baseline.Bike")]   
    Scenario.Total <- l$at_wide[1,c("Baseline.Total")]
  } 
  # Code to correct for rounding error - end
  
  sat$Scenario.Walk <- Scenario.Walk
  sat$Scenario.Bike <- Scenario.Bike
  sat$Scenario.Total <- Scenario.Walk + Scenario.Bike
  
  sat_wide <- rbind(sat, sat_wide)
  sat_wide <- sat_wide[order(sat_wide$Sex, sat_wide$Age) ,]
  
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  # Initialize ubat_wide - user
  # designated baseline active travel
  # time by age and sex
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  ubat_wide <- NULL
  
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  # User Upload Recalculation of
  # Baseline Mean Active Travel Times
  # based on default baseline age-sex
  # ratios
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  
  if (attr(l$scenario, "scenario_type") == "user_defined") {
    bat_means <- at_ratios
    
    bat_means$Baseline.Walk <-
      (at_ratios$Baseline.Walk * 
         at_ratios$Percent / 
         walk_ratio_sum) * 
      Baseline.Walk /
      at_ratios$Percent
    
    bat_means$Baseline.Bike <-
      (at_ratios$Baseline.Bike * 
         at_ratios$Percent / 
         bike_ratio_sum) * 
      Baseline.Bike /
      at_ratios$Percent
    
    bat_means$Baseline.Total <-
      bat_means$Baseline.Walk + 
      bat_means$Baseline.Bike
    
    # Save columns
    ubat_wide <-
      bat_means[, c("Region",
                    "Sex",
                    "Age",
                    "Baseline.Walk",
                    "Baseline.Bike",
                    "Baseline.Total")]
    
    # Add total line
    ubat <- l$at_wide[1, 1:3]
    ubat$Baseline.Walk <- Baseline.Walk
    ubat$Baseline.Bike <- Baseline.Bike
    ubat$Baseline.Total <- Baseline.Walk + Baseline.Bike
    
    ubat_wide <- rbind(ubat, ubat_wide)
    ubat_wide <- ubat_wide[order(ubat_wide$Sex, ubat_wide$Age) ,]
    # Replace default baseline active travel times with user derived baseline
    # replace fat_wide
    l$at_wide <- ubat_wide
  }
  
  # Create dataframe with Age-Sex Walk Bike mean times for display table
  at_sat_wide <- merge(l$at_wide[1:6],
                       sat_wide,
                       by = c("Region", "Sex", "Age"))
  
  # Transit Associated Walking and Cycling for Multiples Scenario
  # Initialize output file
  transit_AT_long <- NULL
  
  # Read in Ratios
  if (attr(l$scenario, "scenario_type")=="multiples") {
    transit_multiple <- l$scenario$multiples[3]
    
    # TransMin units are per week
    transit_AT <- cbind(l$at_wide[1, 1:6], sat[1, 4:6])
    
    transit_AT$Baseline.Transbike  <- 
      l$wbt$BikeTRatio * 
      l$wbt$TransitMin
    
    transit_AT$Baseline.Transwalk  <- 
      l$wbt$WalkTRatio * 
      l$wbt$TransitMin
    
    transit_AT$Baseline.Transtotal <-
      transit_AT$Baseline.Transbike + 
      transit_AT$Baseline.Transwalk
    
    transit_AT$Scenario.Transbike  <-
      l$wbt$BikeTRatio * 
      l$wbt$TransitMin * 
      transit_multiple
    
    transit_AT$Scenario.Transwalk  <-
      l$wbt$WalkTRatio * 
      l$wbt$TransitMin * 
      transit_multiple
    
    transit_AT$Scenario.Transtotal <-
      transit_AT$Scenario.Transbike + 
      transit_AT$Scenario.Transwalk
    
    transit_AT_long <-
      reshape(
        transit_AT,
        varying = c(
          "Baseline.Walk",
          "Baseline.Bike",
          "Baseline.Total",
          "Scenario.Walk",
          "Scenario.Bike",
          "Scenario.Total",
          "Baseline.Transwalk",
          "Baseline.Transbike",
          "Baseline.Transtotal",
          "Scenario.Transwalk",
          "Scenario.Transbike",
          "Scenario.Transtotal"
        ),
        v.names = c("Bike", "Total", "Walk"),
        timevar = "Mode",
        times = c("Baseline", "Scenario", "TransBaseline", "TransitScenario"),
        new.row.names = 1:8,
        direction = "long"
      )
    
    transit_AT_long <- transit_AT_long[, c(1, 4:7)]
    
    names(transit_AT_long)[3:5] <- c("Walk", "Bike", "Total")
    
  } # End of Multiples if function
  
  l <- append(l, list(sat_wide=sat_wide, 
                      at_sat_wide = at_sat_wide, 
                      transit_AT_long = transit_AT_long))
  
  l # return
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# FUNCTION: Physical Activity Quantiles
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' Calculate quantiles
#' 
#' @param l is a list of values from fn_scenario_time() (see above).
#' @return a list with these CHANGES to above inputs: 
#' at_wide
#' and these ADDITIONS:
#' METS_PA, at_temp
fn_pa_quantile <- function(l){
  # Set baseline CV
  l$at_wide$CV <- l$cv[1, "CV"]
  
  # merge with age-sex specific MET weights
  l$at_wide <- merge(l$at_wide,
                     l$METminWalk_Bike, 
                     by = c("Sex","Age"), 
                     all.x = TRUE)
  
  # adds columns to the test dataframe
  # First column after stub: quantiles x number of scenarios * modes
  # temp[,4] = Walkm # temp[,5] = bike, # temp[,6] = total
  at_temp <- l$at_wide
  
  # Scenarios*quintiles*variables
  
  # This replaces scenarios & quantiles loop
  at_temp[(ncol(at_temp) + 1):
            (ncol(at_temp) + l$NQtiles * 2 * 3)] <- 
    as.data.frame(mapply(function(x, y, z) {
      if (x == "Baseline")  { at_temp <- l$at_wide }
      
      if (x == "Scenario")  {
        at_temp[4:6] <- l$sat_wide[4:6]
        at_temp$CV   <-
          -0.0015429 *
          (l$sat_wide[, 6] - l$at_wide[, 6]) +
          l$at_wide$CV
      }
      
      exp(qnorm(y,
                log(at_temp[, z]) - 0.5 *
                  log(1 + at_temp$CV ^ 2),
                sqrt(log(1 + at_temp$CV ^ 2))))
    },
    
    # Arguments:
    x = c(rep("Baseline", l$NQtiles * 3), 
          rep("Scenario", l$NQtiles * 3)),
    y = rep(c(rep(.1, 3), 
              rep(.3, 3), 
              rep(.5, 3), 
              rep(.7, 3), 
              rep(.9, 3)), 2),
    z = c(4, 5, 6)
    ))  # end of mapply
  
  # end of quantile function
  
  # Label the columns
  names(at_temp)[(ncol(l$at_wide) + 1):ncol(at_temp)] <-
    paste0(
      c(rep("Baseline_Q", 15), rep("scenario_Q", 15)),
      c(rep(10, 3), rep(30, 3), rep(50, 3), rep(70, 3), rep(90, 3)),
      c("_walk_min", "_bike_min", "_total_min")
    )
  
  # Weight by METS
  at_temp[(ncol(at_temp) + 1):(ncol(at_temp) + 30)] <-
    as.data.frame(mapply(function(x, y) {
      at_temp[, x] * at_temp$METminWalk / 60
      at_temp[, y] * at_temp$METminBike / 60
      at_temp[, x] * at_temp$METminWalk / 60 + 
        at_temp[, y] * at_temp$METminBike / 60
    },
    x = c(seq(10, 37, by = 3)), y = c(seq(11, 38, by = 3))))
  
  # Label the new columns
  clist <- c(10, 30, 50, 70, 90)
  names(at_temp)[40:69] <-
    c(
      paste0("Baseline_Q", clist, "_walk_MET"),
      paste0("Scenario_Q", clist, "_walk_MET"),
      paste0("Baseline_Q", clist, "_bike_MET"),
      paste0("Scenario_Q", clist, "_bike_MET"),
      paste0("Baseline_Q", clist, "_AT_MET"),
      paste0("Scenario_Q", clist, "_AT_MET")
    )
  
  # Save variables of interest
  METS_PA <-
    at_temp[, c(
      "Age",
      "Sex",
      "Region",
      "Baseline_Q10_AT_MET",
      "Baseline_Q30_AT_MET",
      "Baseline_Q50_AT_MET",
      "Baseline_Q70_AT_MET",
      "Baseline_Q90_AT_MET",
      "Scenario_Q10_AT_MET",
      "Scenario_Q30_AT_MET",
      "Scenario_Q50_AT_MET",
      "Scenario_Q70_AT_MET",
      "Scenario_Q90_AT_MET"
    )]
  
  # Save dataframes
  l <- append(l, list(METS_PA = METS_PA, 
                      at_temp = at_temp))
  
} #end of PA_quantile function

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# FUNCTION: pa paf 
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' Calculate Physical Activity PAF
#' 
#' @param l is a list of values from fn_pa_quantile() (see above).
#' @return a list with no CHANGES to above inputs, and these ADDITIONS:
#' PA_PAF_IHD, PA_PAF_HHD, PA_PAF_Stroke, PA_PAF_Diabetes,
#' PA_PAF_Dementia, PA_PAF_Depression, PA_PAF_ColonCancer, PA_PAF_BreastCancer,
#' PA_PAF_AllCauses
 fn_pa_paf <- function(l) {
  # Set Cause loop
  for (i in 1:length(d$pacauselist)) {
    # Initialize METS temporary dataframe for each cause with quintile METs data
    METS_PA_temp <- l$METS_PA
    
    # determine total METS
    # Make all 9 specific cause based on total METS
    if (d$pacauselist[i] %in% c(
      "Ischemic Heart Disease",
      "Hypertensive Heart Disease",
      "Stroke",
      "Diabetes",
      "Lung Cancer",
                                "Dementia",
                                "Depression",
                                "Colon Cancer",
                                "Breast Cancer",
                                "All Causes")) {
      # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      # Factor in nontravel METS
      # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      # Baseline
      METS_PA_temp[1:16, 4:8]  <-
        l$METS_PA[1:16, 4:8] +  l$nontravelMETS[1:16, 4:8]
      # Scenario
      METS_PA_temp[1:16, 9:13] <-
        l$METS_PA[1:16, 9:13] + l$nontravelMETS[1:16, 4:8]
    }
   
#   print(METS_PA_temp) 
   
     
    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    # Select RR per MET for Cause and its
    # exponent
    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    
    # read RR-PA file
    pa_rr_cause <- subset(l$pa_rr, Cause == d$pacauselist[i])
    
    # merge with relative risks per MET-hr
    # Initiaize PAF data frame
    METS_PAF <- NA
    METS_PAF <-
      merge(METS_PA_temp,
            pa_rr_cause,
            by = c("Sex", "Age"),
            all.x = TRUE)
    
    # Set exponent
    RR_n <-
      ifelse(d$pacauselist[i] == "All Causes", 
             l$PAAllCauseBeta, 
             l$PAChronicBeta)
    
  # Reset total exposure as 0.1 MET if MET is < 2.5
  # Marginal METS no longer uses 2.5 threshold for low dose work around
  # METS_PAF[c(4:13)][METS_PAF[c(4:13)] < 2.5]  <- 0.1
  # Reset total exposure at 10 MET if MET is > 10
  # Use with linear slope between 0 and 10 marginal METS  
    METS_PAF[c(4:13)][METS_PAF[c(4:13)] > 10]  <- 10    
    
    # calculate RR as 5 quintile average
    METS_PAF$RR_PAF <-
      (METS_PAF$RR ^ (METS_PAF$Scenario_Q10_AT_MET ^ RR_n) + 
         METS_PAF$RR ^ (METS_PAF$Scenario_Q30_AT_MET ^ RR_n) + 
         METS_PAF$RR ^ (METS_PAF$Scenario_Q50_AT_MET ^ RR_n) + 
         METS_PAF$RR ^ (METS_PAF$Scenario_Q70_AT_MET ^ RR_n) + 
         METS_PAF$RR ^ (METS_PAF$Scenario_Q90_AT_MET ^ RR_n))/
      (METS_PAF$RR ^ (METS_PAF$Baseline_Q10_AT_MET ^ RR_n) + 
         METS_PAF$RR ^ (METS_PAF$Baseline_Q30_AT_MET ^ RR_n) + 
         METS_PAF$RR ^ (METS_PAF$Baseline_Q50_AT_MET ^ RR_n) + 
         METS_PAF$RR ^ (METS_PAF$Baseline_Q70_AT_MET ^ RR_n) + 
         METS_PAF$RR ^ (METS_PAF$Baseline_Q90_AT_MET ^ RR_n) )
    
    # No change in RR for < 15 year olds
    METS_PAF$RR_PAF[METS_PAF$Age %in% c("00-04", "05-14")] <- 1
    
    # RR for YLD (Same as deaths and YLL for PA, except All Causes)
    METS_PAF$RR_PAF_yld <-
      ifelse(METS_PAF$Cause == "All Causes", 1, METS_PAF$RR_PAF)
    
    # Set PA-RR to 1 if Scenario is LCD
    # Avoids rounding error in baseline and scenario PA times due to ratio
    # calculations
    
    if(attr(l$scenario, "scenario_type")=="evpct") {     
#    if (l$scenario$name %in% c("Low Carbon Driving")) {
      METS_PAF$RR_PAF     <- 1
      METS_PAF$RR_PAF_yld <- 1
    }
    
    # create a new dataframe with cause METS, PAF, Change in BD
    l2 <- list(a=METS_PAF)
    names(l2) <- paste0("PA_PAF_", d$paflist[i])
    l <- append(l, l2)
    rm(l2)
    
  }  # end of Cause loop 
  
  l  # return
} # end pa_paf.function

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# FUNCTION: PM2.5 PAF by cause 
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' Calculates PM 2.5 PAF
#'
#' @param l is a list of values from fn_pa_paf() (see above).
#' @return a list with no CHANGES to above inputs, and these ADDITIONS:
#' PM25_VMT_delta, PM_PAF_IHD, PM_PAF_HHD, PM_PAF_Stroke, PM_PAF_InflamHD,
#' PM_PAF_LungCancer, PM_PAF_RespDx, PM_PAF_AccuteRI, PM_PAF_AllCauses 
fn_pm_paf <- function(l) {

  # Relate change in car VMT to change in PM2.5 ambient concentrations
  # LCD is fixed reduction of 33.5%
  if(attr(l$scenario, "scenario_type")=="evpct") {  
#  if (l$scenario$name %in% c("Low Carbon Driving")) {
 #   l$VMT_pct <- l$LCD_VMT 
l$VMT_pct <- l$EV_PCT 

    #New Code to reflect initial reduction (33.5% and APC in year chosen)    
#    l$VMT_pct <- 100*(l$LCD_VMT/100 * (1 + l$LCD_APC/100)^(l$co2$Year-2015) ) 
    
  } else {
    l$VMT_pct <- 
      100 * 
      (l$bmiles_wide$Baseline.CarDriver -
         l$smiles_wide$Scenario.CarDriver) / 
      l$bmiles_wide$Baseline.CarDriver
  }
  
  # PM2.5 concentration change as a function of car VMT (slope and intercept)
  # l$PM25_VMT <- l$PM25_VMT[l$PM25_VMT$Region == l$fRegion, ]
  l$PM25_VMT_delta <- -(l$PM25_VMT$slope * l$VMT_pct) 
  
  for(i in 1:length(d$pmcauselist)) {

    # Initialize PM2.5 temporary age, sex, region dataframe for each cause 
    PM_temp <- l$METS_PA[1:3] 
    
    # Select CRF for disease [i]
    crf <- d$PM25_RR[which(d$PM25_RR$Cause==d$pmcauselist[i]),]
    
    # merge with age-sex-region matrix
    PM_temp <- merge(PM_temp,crf, all.x=TRUE)
    
    # Calculate the Relative Risk
    PM_temp$RR_PAF <- exp(PM_temp$coefficient* l$PM25_VMT_delta)
    
    # No change in RR for < 30 year olds
    PM_temp$RR_PAF[PM_temp$Age %in% c("00-04", "05-14","15-29") & 
                     d$pmcauselist[i] != "Acute resp infections"] <- 1

    # No change for YLD
    PM_temp$RR_PAF_yld <- 1
    
    # Rename coefficient as RR
    names(PM_temp)[5] <-"RR" 
    
    # create a new dataframe with cause METS, PAF, Change in BD
    l2 <- list(a=PM_temp)
    names(l2) <- paste0("PM_PAF_", d$pmflist[i])
    l <- append(l, l2)
    rm(l2)
    
  } # end of cause loop
  
  l  # return
} # end of pm_paf.function

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# FUNCTION: RTI 
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' Calculates Road Traffic Injuries PAF.
#' 
#' @param l is a list of values from fn_pm_paf() (see above).
#' @return a list with no CHANGES to above inputs, and these ADDITIONS:
#' distance, RTI_PAF_RTI, rti_RR_mode, rti_RR
fn_rti_paf <- function(l) {
  
  p <- l$p
  # Get reshaped scenario miles by mode from wide to long
  sdistance <- l$smiles_long
  sdistance <- sdistance[, 1:3] # get rid of id variable
  
  # Merge Baseline and Scenario distance files
  distance <- merge(l$bdistance, sdistance, by = c("Region", "Mode"))
  
  #Create VMT-PMT columns
  names(distance)[3] <- c("VMT_b")
  names(distance)[4] <- c("VMT_s")
  distance["PMT_b"] <- distance["VMT_b"]
  distance["PMT_s"] <- distance["VMT_s"]
  
  # Consolidate car VMT and PMT for scenario
  distance[3, 5] <- distance[3, 3] + distance[4, 3]
  distance[3, 6] <- distance[3, 4] + distance[4, 4]
  
  # Delete unneeded CarPassenger rows and rename car
  distance <-
    subset(distance, Mode != "CarPassenger" & Mode != "Rail")
  distance <-
    distance[, c("Region", "Mode", "VMT_b", "PMT_b", "VMT_s", "PMT_s")]
  # rename CarDriver to Car
  distance$Mode <- gsub("CarDriver", "Car", distance$Mode)
  
  # Correct bus_VMT by dividing busPMT by occupancy (SF Bay Area = 7)
  distance[2, 3] <- distance[2, 3] / l$bus_occupancy
  distance[2, 5] <- distance[2, 5] / l$bus_occupancy
  
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  # For canned scenarios, baseline and
  # scenario distribution of roadway
  # type by mode is same for all modes
  # except cars
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  
  if (attr(l$scenario, "scenario_type") != "user_defined") {
    bfacility_wide <- l$facility
    sfacility_wide <- l$facility
    names(sfacility_wide)[3:5] <-
      c("local_pct_s", "art_pct_s", "hwy_pct_s")
  }
  
  if (attr(l$scenario, "scenario_type") %in% c("multiples", "absolute_time") | 
      l$scenario$name %in% 
      c("Caltrans Strategic Management Plan (2020)", 
        "CARB Scoping Plan (2030)", 
        "U. S. Surgeon General Recommendation")) {
    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    # recalculate scenario proportions for
    # car given substiution only occurs on
    # local roads and arterials
    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  
    # Create mode miles by facility type matrices
    
    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    # For User Upload, users provide
    # distributions of roadway type by
    # mode for baseline (bfacility_wide)
    # and scenario (sfacility_wide)
    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    distance <-
      merge(distance, bfacility_wide, by = c("Region", "Mode"))
    distance <-
      merge(distance, sfacility_wide, by = c("Region", "Mode"))
    
    local_dist <-
      cbind(distance[,c(3:4)] * distance[, c(7)], 
            distance[,c(5:6)] * distance[, c(10)])
    art_dist   <-
      cbind(distance[, c(3:4)] * distance[, c(8)], 
            distance[, c(5:6)] * distance[, c(11)])
    hwy_dist   <-
      cbind(distance[, c(3:4)] * distance[, c(9)], 
            distance[, c(5:6)] * distance[, c(12)])
    
    # Local car miles PMT_s and VMT_s  (VMT = PMT/car occupancy)
    local_dist[3, 4] <-((1 - p) * local_dist[3, 2]) + 
      p * (sum(local_dist[c(1:3, 6), 2]) - 
      sum(local_dist[c(1:2, 6), 4]))
   
    local_dist[3, 3] <- local_dist[3, 4] / 
      (sum(sdistance[3:4, 3]) / 
         sdistance[3, 3])
    
    # Arterial car miles PMT_s and VMT_s
    art_dist[3, 4] <- ((1 - p) * art_dist[3, 2]) + 
      p * (sum(art_dist[c(1:3, 6), 2]) - 
      sum(art_dist[c(1:2, 6), 4]))
   
    art_dist[3, 3] <-
      art_dist[3, 4] / (sum(sdistance[3:4, 3]) / sdistance[3, 3])
    
    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    # Highway car miles  PMT_s and VMT_s
    # walk, bike, bus             walk,
    # bike, bus                  rail
    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    hwy_dist[3, 4] <-
     hwy_dist[3, 2] - 
      sum(hwy_dist[c(1:2, 6), 4]) + 
      sum(hwy_dist[c(1:2, 6), 2]) - 
      (l$facility[3, 5] * sdistance[6, 3]) + 
      (l$facility[3, 5] * l$bdistance[6, 3])
   
    hwy_dist[3, 3] <-
      hwy_dist[3, 4] / (sum(sdistance[3:4, 3]) / sdistance[3, 3])
    
    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    # Oversubstitution of local roads or
    # arterials requires reallocating car
    # miles across local roads and
    # arterials without changing car
    # highway miles
    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    # This can  happen with USSG type of scenarios
    if (local_dist[3, 3] < 0) {
      local_dist[3, 4]  <-
        (distance[3, 6] - 
           hwy_dist[3, 4]) * local_dist[3, 2] / (local_dist[3, 2] + 
           art_dist[3, 2])
      
      art_dist[3, 4]  <- (distance[3, 6] - hwy_dist[3, 4]) * 
        art_dist[3, 2] / (local_dist[3, 2] + 
                            art_dist[3, 2])
    }
    
    # Total car VMT scenario  and recalculated proportions
    ttl_dist <- local_dist[3, 4] + art_dist[3, 4] + hwy_dist[3, 4]
    distance[3, 10] <- local_dist[3, 4] / ttl_dist
    distance[3, 11] <- art_dist[3, 4] / ttl_dist
    distance[3, 12] <- hwy_dist[3, 4] / ttl_dist
  }
  
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  # For User Upload, users provide
  # distributions of roadway type by
  # mode for baseline (bfacility_wide)
  # and scenario (sfacility_wide)
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  # TODO determine significance of SCS2040
  if (attr(l$scenario, "scenario_type") == "user_defined") {
    distance <- merge(distance, l$bfacility_wide, by = c("Region", "Mode"))
    distance <- merge(distance, l$sfacility_wide, by = c("Region", "Mode"))
  }
  
  if(l$scenario$name %in% c("Sustainable Communities Strategies (2040)", 
#                            "Low Carbon Driving", 
                            "Replacing Short Car Trips with Active Travel") & 
     attr(l$scenario, "scenario_type") == "default"){
    distance <- merge(distance, bfacility_wide, by = c("Region", "Mode"))
    distance <- merge(distance, sfacility_wide, by = c("Region", "Mode"))
  }

# Account for input for LCD 
  
  if(l$scenario$name %in% c("Low Carbon Driving") | 
     attr(l$scenario, "scenario_type") == "evpct"){
    distance <- merge(distance, bfacility_wide, by = c("Region", "Mode"))
    distance <- merge(distance, sfacility_wide, by = c("Region", "Mode"))
  }  
  
    
  # rti is dataframe with baseline injuries
  
  # Loop over severity and roadway type
  sevlist <- c("Fatal", "Serious")
  faclist <- c("Local", "Arterial", "Highway")
  
  l3 <- list()  # temp
  SiN <- l$SiN
  
  # Begin severity loop
  for (i in 1:length(sevlist)) {

    # Subset baseline injuries by severity
    rti_sev <- subset(l$rti, Severity == sevlist[i])
    
    for (j in 1:length(faclist)) {
      # Subset baseline injuries by roadway type
      rti_temp <- subset(rti_sev, Roadway == faclist[j])
      
      # Initialize tempoary distance file for looping
      distance$facility <- faclist[j]
      distance_temp <- distance
      
      # calculate facility specific VMT and PMT
      if (distance$facility[1] == "Local") {
        distance_temp[3] <- distance[3] * distance[7]  #VMT_b
        distance_temp[4] <- distance[4] * distance[7]  #PMT_b
        distance_temp[5] <- distance[5] * distance[10] #VMT_s
        distance_temp[6] <- distance[6] * distance[10] #PMT_s
      }
      
      if (distance$facility[1] == "Arterial") {
        distance_temp[3] <- distance[3] * distance[8]
        distance_temp[4] <- distance[4] * distance[8]
        distance_temp[5] <- distance[5] * distance[11]
        distance_temp[6] <- distance[6] * distance[11]
      }
      if (distance$facility[1] == "Highway") {
        distance_temp[3] <- distance[3] * distance[9]
        distance_temp[4] <- distance[4] * distance[9]
        distance_temp[5] <- distance[5] * distance[12]
        distance_temp[6] <- distance[6] * distance[12]
      }
      
      # merge with distances
      rti_temp <-
        merge(
          rti_temp,
          distance_temp,
          by.x = c("VictimMode", "Region"),
          by.y = c("Mode", "Region")
        )
      
      # reorder rows
      rti_temp <- rti_temp[c(6, 1, 2, 3, 5, 4), ]
      
      # Delete uncessary columns
      rti_temp <- rti_temp[, -c(1:4, 16:22)]
      rownames(rti_temp) <-
        c("Walk", "Bike", "Bus", "Car", "Truck", "Motorcycle")
      
      # Scenario Injuries in Victim-Striking Vehicle Matrix
      # Looping
      # Begin row loop
      for (k in 1:6) {
        # Begin column loop
        for (m in 1:6) {
          # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
          # scenario VMT-s * scenario PMT-V
          # Inj-baseline/ (baseline VMT-S *
          # baseline PMT-V)
          # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
          
          rti_temp[k, m + 11] <- ((rti_temp[m, 10] * rti_temp[k, 11]) ^ SiN) * 
            rti_temp[k, m] / 
            ((rti_temp[m, 8] * rti_temp[k, 9]) ^ SiN)
        }
      } # Striking vehicle loop
      
      # No other vehicle
      for (k in 1:6) {
        rti_temp[k, 18] <-
          ((rti_temp[k, 11] / rti_temp[k, 9]) ^ SiN) * rti_temp[k, 7]
        
      } # End No Other Vehicle
      
      # Name the Scenario Injuires columns
      names(rti_temp)[12:18] <-
        c("walk_s",
          "bike_s",
          "bus_s",
          "car_s",
          "truck_s",
          "motorcycle_s",
          "NOV_s")
      
      # Internally label data frame by severity and roadway type
      rti_temp$Region <- l$fRegion
      rti_temp$Severity <- sevlist[i]
      rti_temp$Roadway  <- faclist[j]
      rti_temp$VictimMode <- row.names(rti_temp)
      rti_temp <- rti_temp[, c(19:22, 1:18)]
      
      # create a new dataframe
      l2 <- list(a=rti_temp)
      names(l2) <- paste0("PAF_RTI_", sevlist[i], "_", faclist[j])
      l3 <- append(l3, l2)
      rm(l2)

      
    } # end facility type loop
  }  #end severity loop

  # Create Summary data by aggregating over facility type
  
  # Combine files over severity and roadway type
  rti_RR <-
    rbind(
      l3$PAF_RTI_Fatal_Local,
      l3$PAF_RTI_Fatal_Arterial,
      l3$PAF_RTI_Fatal_Highway,
      l3$PAF_RTI_Serious_Local,
      l3$PAF_RTI_Serious_Arterial,
      l3$PAF_RTI_Serious_Highway
    )
  
  rti_RR$vinjuries_baseline <- rti_RR$walk +   
    rti_RR$bike +   
    rti_RR$car +   
    rti_RR$bus +   
    rti_RR$truck +   
    rti_RR$motorcycle +   
    rti_RR$NOV
  
  rti_RR$vinjuries_scenario <- rti_RR$walk_s + 
    rti_RR$bike_s + 
    rti_RR$car_s + 
    rti_RR$bus_s + 
    rti_RR$truck_s + 
    rti_RR$motorcycle_s +
    rti_RR$NOV_s
  
  # Sum over Roadway types for Fatal injuries
  rti_RR_mode <-
    aggregate(rti_RR[, c("VMT_b",
                         "VMT_s",
                         "vinjuries_baseline",
                         "vinjuries_scenario")],
              list(rti_RR$Region, rti_RR$Severity, rti_RR$VictimMode),
              FUN = sum)
  names(rti_RR_mode)[1:3] <- c("Region", "Severity", "VictimMode")
  rti_RR_mode <-
    rti_RR_mode[order(rti_RR_mode$Severity, rti_RR_mode$VictimMode) ,]
  
  # Bring in population for calculation of injury rates
  Population <- sum(l$age_sex_region_year$Population)
  
  # Create injury rate (IR) per 10E7 miles and RR
  rti_RR_mode$IR_b <-
    10000000 * rti_RR_mode$vinjuries_baseline / (rti_RR_mode$VMT_b * Population)
  rti_RR_mode$IR_s <-
    10000000 * rti_RR_mode$vinjuries_scenario / (rti_RR_mode$VMT_s * Population)
  
  # Create overall RR by Severity
  rti_RR_mode[13, 4:9] <-
    colSums(rti_RR_mode[1:12, 4:9][rti_RR_mode$Severity == "Fatal", ])
  rti_RR_mode[13, 1:3] <- c(l$fRegion, "Fatal", "All Modes")
  rti_RR_mode[14, 4:9] <-
    colSums(rti_RR_mode[1:12, 4:9][rti_RR_mode$Severity == "Serious", ])
  rti_RR_mode[14, 1:3] <- c(l$fRegion, "Serious", "All Modes")
  rti_RR_mode$RR <-
    rti_RR_mode$vinjuries_scenario / rti_RR_mode$vinjuries_baseline
  RTI_RR   <- rti_RR_mode[13, 10]
  RTI_RR_yld <- rti_RR_mode[14, 10]
  
  # Create PAF file for burden of disease calculation
  RTI_PAF_RTI <- l$at_wide[c("Region", "Sex", "Age")]
  RTI_PAF_RTI$Cause <- "Road Traffic Injuries"
  RTI_PAF_RTI$RR_PAF <- RTI_RR
  RTI_PAF_RTI$RR_PAF_yld <- RTI_RR_yld
  
  l <- append(l, list(
    distance = distance,
    RTI_PAF_RTI = RTI_PAF_RTI,
    rti_RR_mode = rti_RR_mode,
    rti_RR = rti_RR
  ))
  
} # rti_paf.function 


# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# FUNCTION: Burden of Disease 
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' Calculates Burden of Disease.
#' 
#' @param l is a list of values from fn_rti_paf() (see above).
#' @return a list with these CHANGES to above inputs: 
#' cbd, PA_PAF_IHD, PA_PAF_HHD, PA_PAF_Stroke,
#' PA_PAF_Diabetes, PA_PAF_Dementia, PA_PAF_Depression, PA_PAF_ColonCancer,
#' PA_PAF_BreastCancer, PA_PAF_AllCauses, PM_PAF_IHD, PM_PAF_HHD,
#' PM_PAF_Stroke, PM_PAF_InflamHD, PM_PAF_LungCancer, PM_PAF_RespDx,    
#' PM_PAF_AccuteRI, PM_PAF_AllCauses, RTI_PAF_RTI  
#' and these ADDITIONS:
#' PA+PM_PAF_IHD, PA+PM_PAF_HHD, PA+PM_PAF_Stroke, PA+PM_PAF_All Causes, LungCancer
#' cause_sum     
fn_burden_causes <- function(l) {

  # Placeholder for addiitonal risk adjuster (e.g. rece/ethnicity, income)

  # Merge files
  l$cbd <- merge(l$cbd,
               l$riskadjust, 
               by=c("Region","Age","Sex", "Cause"), all.x=TRUE)
  
  #Adjust CBD (deaths, yll, yld, dalys)
  l$cbd[7:10] <- l$cbd[7:10]* l$cbd$RRadj
  
  # Merge with population file
  l$cbd <- merge(l$cbd,
                 l$age_sex_region_year, 
                 by=c("Sex","Age","Region"), all.x=TRUE)
  
  # Merge with APC file 
  l$cbd <- merge(l$cbd, 
                 l$apc, by=c("Sex","Age","Cause"), all.x=TRUE)
  
  # rcauselist <- c("Road Traffic Injuries")
  # rflist <- c("RTI")
  # plist <- c("PA", "PM", "RTI", "PA+PM")
  
  # Pathway loop
  for (i in 1:length(d$plist)) {
    if (d$plist[i] == "PA")  {
      causelist <- d$pacauselist
      flist <- d$paflist
    } else
      if (d$plist[i] == "PM")  {
        causelist <- d$pmcauselist
        flist <- d$pmflist
      } else
        if (d$plist[i] == "RTI") {
          causelist <- d$rcauselist
          flist <- d$rflist
        } else
          if (d$plist[i] == "PA+PM") {
            causelist <- d$papmcauselist
            flist <- d$papmflist
          }
    
    # Causes within pathways
    for (j in 1:length(causelist)) {

      cause_temp <- subset(l$cbd, Cause == causelist[j])
      
      # Non-multiplicative RR
      if (d$plist[i] %in% c("PA", "PM", "RTI"))
      { 

        cause_temp <-
          merge(l[[paste0(d$plist[i], "_PAF_", flist[j])]],
                cause_temp,
                by = c("Region", "Sex", "Age", "Cause"))
        
      } else

        # Calculate Multiplicative RR
        if (d$plist[i] == "PA+PM") {
          
          # TODO fix papmflist's All Causes (note the space, hence gsub below)
          PM_file <- l[[paste0("PM_PAF_",
                               d$papmflist[j])]][1:16, 1:22]

          PA_file <- l[[paste0("PA_PAF_", 
                               d$papmflist[j])]][1:16,c(1:4, 16:17)]

          cause_temp <-
            merge(PA_file, 
                  PM_file, 
                  by = c("Region", "Sex", "Age", "Cause"))
          
          # Compute Multiplicative RR
          cause_temp$RR_PAF <-
            cause_temp$RR_PAF.x * cause_temp$RR_PAF.y
          cause_temp$RR_PAF_yld <-
            cause_temp$RR_PAF_yld.x * cause_temp$RR_PAF_yld.y
        }
      
      
      # Burden of Disease
      cause_temp$bd.deaths <-
        cause_temp$Population * 
        (cause_temp$deaths / cause_temp$pop) * 
        (1 - cause_temp$APC / 100) ^ (cause_temp$Year.y - cause_temp$Year.x)
      
      cause_temp$bd.yll <-
        cause_temp$Population * 
        (cause_temp$yll / cause_temp$pop) * 
        (1 - cause_temp$APC / 100) ^ (cause_temp$Year.y - cause_temp$Year.x)
      
      cause_temp$bd.yld <-
        cause_temp$Population * 
        (cause_temp$yld / cause_temp$pop) * 
        (1 - cause_temp$APC / 100) ^ (cause_temp$Year.y - cause_temp$Year.x)
      
      cause_temp$bd.daly <- cause_temp$bd.yll + cause_temp$bd.yld
      
      # Calculation (1-PAF)* rate * (1-APC)^ Y(horizon)-Year(2010)
      # Change sign so BD is negative when RR < 1
      
      cause_temp$change.deaths <-
        -1 * (1 - cause_temp$RR_PAF) * cause_temp$bd.deaths
      cause_temp$change.yll <-
        -1 * (1 - cause_temp$RR_PAF) * cause_temp$bd.yll
      cause_temp$change.yld <-
        -1 * (1 - cause_temp$RR_PAF_yld) * cause_temp$bd.yld
      cause_temp$change.daly <-
        cause_temp$change.yll + cause_temp$change.yld
      
      # Sum up columns and label
      cause_temp$Pathway <- d$plist[i]
      cause_temp[17, c(
        "Population",
        "bd.deaths",
        "bd.yll",
        "bd.yld",
        "bd.daly",
        "change.deaths",
        "change.yll",
        "change.yld",
        "change.daly"
      )] <-
        colSums(cause_temp[1:16, c(
          "Population",
          "bd.deaths",
          "bd.yll",
          "bd.yld",
          "bd.daly",
          "change.deaths",
          "change.yll",
          "change.yld",
          "change.daly"
        )], na.rm = TRUE)
      
      cause_temp[17, c("Year.x", "Year.y", "Geography", "Pathway")]  <-
        cause_temp[1, c("Year.x", "Year.y", "Geography", "Pathway")]
      cause_temp[17, 1:4] <- c(l$fRegion, "Both", "Total", causelist[j])
      
      # Create PAF
      cause_temp$PAF.deaths <-
        cause_temp$change.deaths / cause_temp$bd.deaths
      cause_temp$PAF.yll    <-
        cause_temp$change.yll / cause_temp$bd.yll
      cause_temp$PAF.yld    <-
        cause_temp$change.yld / cause_temp$bd.yld
      cause_temp$PAF.daly   <-
        cause_temp$change.daly / cause_temp$bd.daly
      
      # Rename Sex (Order: Male, Female, Both like spreadsheet)
      cause_temp$Sex[cause_temp$Sex == "2"] <- "F"
      cause_temp$Sex[cause_temp$Sex == "1"] <- "M"
      
      
      # Create Summary table of each disease and pathway
      # First row of cause_sum is repeated in initialization, deleted later
      last_row <-
        cause_temp[17, c(
          "Pathway",
          "Region",
          "Sex",
          "Age",
          "Cause",
          "Geography",
          "Population",
          "bd.deaths",
          "bd.yll",
          "bd.yld",
          "bd.daly",
          "change.deaths",
          "change.yll",
          "change.yld",
          "change.daly",
          "PAF.deaths",
          "PAF.yll",
          "PAF.yld",
          "PAF.daly"
        )]
      if (i == 1 & j == 1) {
        cause_sum <- last_row
      }
      cause_sum <- rbind(cause_sum, last_row)
      
      # Add data to output list.
      l2 <- list(a=cause_temp)  # create temp list named l2
      nm <- paste0(d$plist[i], "_PAF_", flist[j])  # create name for variable.
      names(l2) <- nm  # assign variable name to our temp list l2
      if(names(l2) %in% names(l)){
        # replace existing values in l
        l[[nm]] <- l2[[nm]]
      } else {
        # Or add new values to l
        l <- append(l, l2)
      }
      rm(l2, cause_temp)  # Garbage Collection
      
    } # End Causes loop
    
  } # End pathway loop
  
  # replace NaN values with 0.
  if(T %in% is.nan(cause_sum)){
    # depends on is.nan method for data.frame being defined first.
    cause_sum[is.nan(cause_sum)] <- 0
  }
  
  l <- append(l, list(cause_sum = cause_sum))
  
} # End burden_causes.function

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# FUNCTION: Monetization of Health Outcomes 
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' Calculates monetization.
#' 
#' @param l is a list of values from fn_burden_causes() (see above).
#' @return a list with these CHANGES to above inputs: 
#' cause_sum
#' and no ADDITIONS.
fn_monetization <- function(l){
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  # Create summary table with all causes
  # and pathways for Cost monetization,
  # delete redundant first rows of input
  # tables
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  
  # if(l$fGeography == "San Francisco") browser()
  
  l$cause_sum <- l$cause_sum[2:25,] 

# print(l$cause_sum)
  
  # Merge summary disease file with COI file
  l$cause_sum <- merge(l$cause_sum, 
                       d$coi, 
                       by = c("Cause"), 
                       all.x = TRUE)
  # Population  *  cost per capita * PAF
  l$cause_sum$coi <-
    l$cause_sum$Population * 
    l$cause_sum$PerCapita2010USD * 
    l$cause_sum$PAF.daly
  
  # Deaths * Value of a statistical life
  l$cause_sum$vsl <- l$cause_sum$change.deaths * l$VSL
  
  l  # return
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# FUNCTION: Car Carbon Emissions 
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' @param l is a list of values from fn_monetization() (see above).
#' @return a list with these CHANGES to above inputs: 
#' co2
#' and no ADDITIONS.
fn_carbon_emissions <- function(l){
  # Add row for 2010 emsision rate applied to 2040 population
  l$co2 <- rbind(l$co2[1,], l$co2)

#  l$co2temp <- rbind(l$co2[1:2,],l$co2[2,])
#  l$co2     <- if(is.na(l$co2[3,1])) {l$co2temp} else 
#    if(!is.na(l$co2[3,1])){l$co2}
  
  
  # Difference in percapita car VMT between baseline and scenario
  l$co2$carVMTbase <- l$bdistance[l$bdistance$Mode == "CarDriver", 3]
  
  # l$co2$carVMTscenario <- sdistance[sdistance$Mode=="CarDriver",3]
  l$co2$carVMTscenario <-
    l$smiles_long[l$smiles_long$Mode == "CarDriver", 3]
  
  # Calculate total population in base year, 2010 and year of choice
  l$co2$Population <- l$age_sex_pop[l$age_sex_pop$Age == "Total", 4]
  l$co2[2:3, "Population"] <-
    colSums(l$age_sex_region_year["Population"])
  
  # Aggregate CO2 (10^6 g/metric ton) and Percapita CO2 (metric ton/person)
  l$co2$MMTY_base     <-
    (l$co2$CO2g_mi * 
       l$co2$carVMTbase * 
       l$co2$Population) / 
    (1000000 * 1000000)
  
  # low carbon driving vs rest.
  if(attr(l$scenario, "scenario_type")=="evpct") {  
#    if(l$scenario$name %in% c("Low Carbon Driving")) {
    l$co2$MMTY_scenario <-
#      (1 - l$LCD_VMT/100) *  
      (1 - l$VMT_pct/100) *        
      (l$co2$CO2g_mi * 
         l$co2$carVMTscenario * 
         l$co2$Population)/
      (1000000*1000000)
  } else {
    l$co2$MMTY_scenario <- 
      (l$co2$CO2g_mi * 
         l$co2$carVMTscenario * 
         l$co2$Population)/
      (1000000*1000000)
  }
  
  l$co2$Aggregate_MMT_dif <-  l$co2$MMTY_scenario - l$co2$MMTY_base
  
  l$co2$Percapita_MT_dif <-
    1000000 * 
    (l$co2$MMTY_scenario - l$co2$MMTY_base) / 
    l$co2$Population
  
  l  # return
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# FUNCTION: Units Conversion
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' @param l is a list of values from fn_carbon_emissions() (see above).
#' @return a list with these CHANGES to above inputs: TODO
fn_units_conversion <- function(l){

  # Miles/kilometers conversion, Baseline[3] & Scenario[4]
  if (l$td_unit == "Kilometers") {
    l$bs_distance_mode[3] <- l$bs_distance_mode[3] * 1.60934
    l$bs_distance_mode[4] <- l$bs_distance_mode[4] * 1.60934
  }  # Miles is default.
  
  # Miles traveled per year, week, or day (default is year)
  l$bs_distance_mode[3] <- switch(
    l$td_time,
    "Week" = l$bs_distance_mode[3]/52,
    "Day" = l$bs_distance_mode[3]/365,
    l$bs_distance_mode[3]
  )  # Year is default.
  
  l$bs_distance_mode[4] <- switch(
    l$td_time,
    "Week" = l$bs_distance_mode[4]/52,
    "Day" = l$bs_distance_mode[4]/365,
    l$bs_distance_mode[4]
  )  # Year is default.
  
  # Centrality (swaps out at_sat_wide (mean) for at_temp (median Q50)
  if(l$centrality == "Median") {
    l$at_sat_wide[4:9] <- sapply(
      X = c(16, 17, 18, 31, 32, 33),  # The select columns in l$at_temp
      FUN = function(x) {
        l$at_temp[x]
      }
    )
  }  # Mean is default.
  
  # Minutes of Physical Activity conversion (Min per day or min per week)
  if (l$at_time_unit == "Day") {
    l$at_sat_wide[4:9] <- sapply(
      4:9,
      function(x){
        l$at_sat_wide[x]/7
      }
    )
  }  # Week is default.

  l  # return
}