# Static Shiny Functions for ITHIM TOOL
# June 2019 - Tomari
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ////////////////////////////////////////////////////////////////////////////

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# FUNCTION: User Data Threshold Verification
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# TODO

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# FUNCTION: Split Scenarios & Baselines
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' Takes a csv of scenarios/baselines and splits it. This works for both default 
#' (i.e. canned ITHIM scenarios/baselines) and user uploaded data. For the most 
#' part, error  checking and data verification of the user data does not occur 
#' here. See fn_user_data_verify(). However, we do check for headers right here.
#' In fact, if there are extraneous columns, they are excised.
#'
#' @param df
#' @return a list of scenarios with each scenario containing its own list of 
#' two data.frames including "Distances" and "Proportion of Vehicle Miles by 
#' Mode and Facility Type".
fn_narrativeSplitter <- function(df){  # former arg: origin
  tryCatch(
    error = function(cnd){
      return(
        # paste0("User Error: ",
        #             "The data (csv) has some flaw.")
        d$user_error[which(d$user_error=="UE1"), "Message"]
        )
    },{
      # check to make sure headers are correct
      headers <- c(
        "Region",
        "Item_Name",
        "Scenario_ID",
        "Mode",
        "Item_Result"
      )
      
      # If the df is 5 columns, its a user upload.
      if(ncol(df) != 5){
        # add Strata
        headers <- c(headers[1:4], "Strata", headers[5])
        # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        # Note, the purpose of this
        # conditional is to add a check for  a
        # column named "Strata". However, this
        # omits the possibility that user
        # upload is 5 columns only! A second
        # check is therefore added in the
        # fn_user_data_verify
        # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      }
      
      checklist <- sapply(headers, function(x){
        if(x %in% names(df)) 
          T 
        else
          F
      })
      
      if (F %in% checklist)
        return(
          fn_keyworder(
            x = d$user_error[which(d$user_error == "UE2"), "Message"],
            lookup = data.frame(
              key = "{Headers}",
              value = paste(names(checklist)[which(checklist == F)], 
                            collapse = ", "),
              stringsAsFactors = F
              )
          )  # end of fn_keyworder
        )
      
      # Subset the desired columns
      df <- df[,headers]
      
      # First, split by Scenario_ID
      narratives <- split(df, df$Scenario_ID)
      
      # then split by item name
      narratives <- lapply(narratives, function(x) split(x, x$Item_Name))
      
      # Remove (numbered) rownames
      for(i in seq_along(narratives)){
        for(j in seq_along(narratives[[i]])){
          rownames(narratives[[i]][[j]]) <- NULL
        }
      }
      
      # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      # Rename list items...
      # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      # Distances
      for (i in seq_along(narratives)) {
        names(narratives[[i]])[which(
          grepl("distance", 
                names(narratives[[i]]), 
                ignore.case = T))] <- "Distances"
        
      }
      
      narratives  # return
    }
  )
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# FUNCTION: User Data Verification
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' Verifies user uploaded data for appropriate content. Amongst things we were
#' concerned with include the presence of at least two narratives (i.e. we need
#' at least one baseline and one scenario); we need within each narrative a
#' Per Capita Mean Daily Travel Distance & Proportion of Vehicle Miles by Mode
#' and Facility Type data.frames; we need to check for missing or structurally
#' incorrect values for each Item_Result column (and here we include conditional
#' decision making based on which of the above data.frames are being examined).
#' It should be noted that, where appropriate, missing values will be replaced 
#' by Baseline 2010 values, but this occurs in an observeEvent() in 
#' tool_modules.R. Finally, we also need to check to make sure that an 
#' appropriate Region has been provided.
#' 
#' @param nar is a list object with narratives produced by fn_narrativeSplitter
#' @return a list with the continue (T/F) + character vector with message(s)
fn_user_data_verify <- function(nar){
  # initialize the return object
  out <- list(
    continue = F,
    msg = NA
  )
  
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  # 1. Previous Error ----
  # Verify that nar is list; if its a
  # character, that means that
  # fn_narrativeSplitter threw an error
  # (eg the tryCatch statement)
  if(class(nar) == "character") {
    # In other words, a fatal error has occurred.
    out$msg <- nar
    return(out)  # returns previous error message.
  }

  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  # 1.5 Missing Strata ----
  # Note, this is not an issue with
  # other columns, as they are accounted
  # for properly in the
  # fn_narrativeSplitter. Indeed, here
  # we find that a 5 column data.frame
  # was passed to fn_user_data_verify.
  if(!("Strata" %in% names(nar[[1]][[1]]))){
    out$msg <- fn_keyworder(
      x = d$user_error[which(d$user_error == "UE2"), "Message"],
      lookup = data.frame(
        key = "{Headers}",
        value = "Strata",
        stringsAsFactors = F
      )
    )  # end of fn_keyworder
  
    return(out)
  }
  
  
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  # 2. Narratives 2+ ----
  # Check that there are at least 2 narratives
  if(length(nar) < 2) {
    # In other words, a fatal error has occurred.
    out$msg <- d$user_error[which(d$user_error=="UE3"), "Message"]
    return(out)
  }
  
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  # 2.5. Misspelled or Missing Scenario_ID ----

  # Get nrow of each narrative's data.frames.
  tmp <- lapply(nar, function(x){
    unname(sapply(x, nrow, USE.NAMES = F))
  })
  
  # Check if there are two data.frames, one w/ 7, the other w/ 9 rows.
  tmp2 <- lapply(tmp, function(x){
    F %in% (8 %in% x & 9 %in% x) & length(x) == 2
  })
  
  # Check if missing/misspelled name for Scenario_ID
  if("" %in% names(tmp)) {
    # Missing Scenario_ID value.
    
    # Name of Scenario with missing value. Grab first if multiple.
    scenarioID <- names(tmp2[tmp2==T][1])
    
    out$msg <- fn_keyworder(
      x = d$user_error[which(d$user_error=="UE13"), "Message"],
      lookup = data.frame(
        key = "{Scenario_ID}",
        value = scenarioID,
        stringsAsFactors = F
      )
    )  # end of fn_keyworder
    
    return(out)
    
  } else if (T %in% tmp2) {
    # Misspelled Scenario_ID value.
    
    # Name of Scenario with misspelled value. Grab first if multiple.
    scenarioID <- names(tmp2[tmp2==T][1])
    
    out$msg <- fn_keyworder(
      x = d$user_error[which(d$user_error=="UE14"), "Message"],
      lookup = data.frame(
        key = "{Scenario_ID}",
        value = scenarioID,
        stringsAsFactors = F
      )
    )  # end of fn_keyworder
    
    return(out)
  }
  
  rm(tmp, tmp2)
  
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  # 3. Dist & Prop ----
  # Check to make sure both distances and proportions are present for each 
  # narrative
  msg <- lapply(seq_along(nar), function(x){
    # obtain Scenario_ID
    scenarioID <- names(nar[x])
    
    # make sure each scenario has only one element with the name "distance"
    if(length(grep("distance", 
                   names(nar[[scenarioID]]),
                   ignore.case = T)) != 1)
      return(
        fn_keyworder(
          x = d$user_error[which(d$user_error == "UE4"), "Message"],
          lookup = data.frame(
            key = "{Scenario_ID}",
            value = scenarioID,
            stringsAsFactors = F
          )
        )  # end of fn_keyworder
      )
    
    # make sure each scenario has only one element with the name "proportion"
    else if(length(grep("proportion", 
                        names(nar[[scenarioID]]),
                        ignore.case = T)) != 1)
      return(
        fn_keyworder(
          x = d$user_error[which(d$user_error == "UE5"), "Message"],
          lookup = data.frame(
            key = "{Scenario_ID}",
            value = scenarioID,
            stringsAsFactors = F
          )
        )  # end of fn_keyworder
      )
    
    # return NULL if no issues.
    else
      NULL  
  })
  
  # Throw Error message (i.e. return early with a msg).
  if(T %in% sapply(msg, function(x) is.character(x))) {
    # Remove NULL values from msg.
    msg <- msg[sapply(msg, 
                      function(x) !is.null(x))]
    # In other words, a fatal error has occurred.
    out$msg <-
      paste(
        msg,
        collapse =  "<br>"
      )
    return(out)
  }
  rm(msg)
  
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  # Check for errors in Per Capita Mean Daily Travel Distance
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  # 4. Dist All Modes ----
  # Are all modes accounted for?
  modes <- c("Bike",
             "Bus",
             "CarDriver",
             "CarPassenger",
             "Motorcycle",
             "Rail",
             "Walk",
             "Truck")
  
  modes_present <- lapply(nar, function(x){
    # check each narrative for missing modes
    # tmp is a list, with each element composed of a series of
    # NULL or character values
    
    # extract distances df from x (a list object)
    dist <- x[[grep("distance", x, ignore.case = T)[1]]]
    
    if(F %in% (unique(dist$Mode) %in% modes))
      return(
        fn_keyworder(
          x = d$user_error[which(d$user_error == "UE6"), "Message"],
          lookup = data.frame(
            key = "{Scenario_ID}",
            value = dist$Scenario_ID[1],
            stringsAsFactors = F
            )
        )  # end of fn_keyworder
      )
    
    # run functional to check if all modes present. Store errors/NULL in tmp.
    msg <- lapply(modes, function(y){
      # T or F, length is equal to one
      if(nrow(dist[dist$Mode==y,]) != 1)
        fn_keyworder(
          x = d$user_error[which(d$user_error == "UE7"), "Message"],
          lookup = data.frame(
            key = c("{Scenario_ID}", "{Mode}"),
            value = c(dist$Scenario_ID[1], y),
            stringsAsFactors = F
          )
        )  # end of fn_keyworder
      else
        # If no problems, NULL
        NULL
    })
    
    if(T %in% sapply(msg, function(y) is.character(y))){
      # Remove NULL values from msg.
      msg <- msg[sapply(msg, 
                        function(x) !is.null(x))]
      # a mode is missing
      return(
        paste(
          msg,
          collapse = "<br>"
        )
      )  
    } else
      # If no problems, NULL
      return(NULL)  # all modes accounted for
  })  # end  modes_present <- lapply
  
  # Throw Error message (i.e. return early with a msg).
  if (T %in% sapply(modes_present, function(x) is.character(x))) {
    modes_present <- modes_present[sapply(modes_present, 
                         function(x) !is.null(x))]
    # In other words, a fatal error has occurred.
    out$msg <- paste(modes_present,
                     collapse = "<br>")
    return(out)
  }
  rm(modes_present)
  
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  # 5. Dist Blanks 1 ----
  # Are there unacceptable blanks?
  # Bus, Motorcycle, & Truck can be blank, but substituted by Baseline 2010.
  
  # Modes which must not be blank.
  filled <- c(
    "Bike",
    "CarDriver",
    "CarPassenger",
    "Walk"
  )
  
  filled_dist <- lapply(nar, function(x){
    # get Distances df from x (which is a list)
    dist <- x[[grep("distance", x, ignore.case = T)[1]]]
    
    # verify that Item_Result for each "filled' is of right type.
    msg <- sapply(filled, function(y){
      item_result_y <- dist$Item_Result[dist$Mode==y]
      
      # Error if value is NA, NaN, NULL, infinite, or non-numeric.
      if(is.nan(item_result_y) | 
         is.na(item_result_y) |
         !(is.numeric(item_result_y)) |
         is.infinite(item_result_y)
         )
        fn_keyworder(
          x = d$user_error[which(d$user_error == "UE7"), "Message"],
          lookup = data.frame(
            key = c("{Scenario_ID}", "{Mode}"),
            value = c(dist$Scenario_ID[1], y),
            stringsAsFactors = F
          )
        )  # end of fn_keyworder
      else
        # If no problems, NULL
        NULL
    })
    
    if (T %in% sapply(msg, function(y) is.character(y))){
      msg <- msg[sapply(msg, 
                        function(x) !is.null(x))]
      # Error was present.
      return(paste(msg, collapse = "<br>"))
    } else
      # If no problems, NULL
      NULL
  })  # end blank_dist_modes <- lapply
  
  # Throw Error message (i.e. return early with a msg).
  if(T %in% sapply(filled_dist, is.character)){
    filled_dist <- filled_dist[sapply(filled_dist, 
                                      function(x) !is.null(x))]
    # In other words, a fatal error has occurred.
    out$msg <- paste0(
      filled_dist,
      collapse = "<br>"
    )
    return(out)
  }
  
  rm(filled_dist, filled)
  
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  # 6. Dist Blanks 2 ----
  # Check for blanks in Bus/Motorcycle/Truck modes in Distances
  # If blank, add warning. Replacements occur in reactive fn `run`
  
  # Modes which may be blank.
  may_be_blank <- c("Bus", "Motorcycle", "Truck", "Rail")
  
  # Just check for blanks, then create warning.
  # mbb_dist is short for may be blank distances
  mbb_dist <- lapply(nar, function(x) {
    # get Distances df from x (which is a list)
    dist <- x[[grep("distance", x, ignore.case = T)[1]]]
    
    msg <- sapply(may_be_blank, function(y){
      # retrieve value for Item_Result
      item_result_y <- dist$Item_Result[dist$Mode==y]

      # check to see if its blank (which is fine.)
      if(is.nan(item_result_y) | 
         is.na(item_result_y) |
         !(is.numeric(item_result_y)) |
         is.infinite(item_result_y)){
        
        fn_keyworder(
          x = d$user_error[which(d$user_error == "UE8"), "Message"],
          lookup = data.frame(
            key = c("{Scenario_ID}", "{Mode}"),
            value = c(dist$Scenario_ID[1], y),
            stringsAsFactors = F
          )
        )  # end of fn_keyworder
        
      } else
        # If no problems, NULL
        NULL
    })
    
    if(T %in% sapply(msg, function(y) is.character(y))){
      # Remove NULL values from msg.
      msg <- msg[sapply(msg, 
                        function(x) !is.null(x))]
      
      # Return warning(s) if present in msg.
      paste0(
        msg,
        collapse = "<br>"
      )
    } else
      # If no problems, NULL
      NULL
  })  # end of lapply for mbb_dist
  
  # If blanks were detected...
  if(T %in% sapply(mbb_dist, function(y) is.character(y))){
    
    # retrieve any warning messages
    mbb_dist <- mbb_dist[sapply(mbb_dist, 
                                function(x) !is.null(x))]
    
    # Add the warning message (of changed blanks) to out$msg
    out$msg <- paste0(
      mbb_dist,
      collapse = "<br>"
    )
    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    # Note, no return() is triggered,
    # because these blanks are just
    # Warnings.
    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    
  }  # end of if statement triggered by presence of Warnings about blanks.
  
  rm(mbb_dist, may_be_blank)
  
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  # Check for errors in Proportion of Vehicle Miles by Mode and Facility type
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  # 7. Prop All Strata ----
  # Check to make sure all Strata/Mode are present
  # (i.e. not Item_Result yet)
  
  # vector of strata
  strata <- c(
    "arterial",
    "highway",
    "local"
  )
  
  modes <- c(
    "Bus",
    "Car",
    "Truck"
  )
  
  # detect mode/strata in Proportion of Vehicle Miles by Mode and Facility Type
  all_strata <- lapply(nar, function(x){
    # extract Proportion... data.frame
    prop <- x[[grep("proportion", x, ignore.case = T)[1]]]
    # check to make sure the proper Strata/Mode configuration is present.
    msg <- sapply(modes, function(y){
      sapply(strata, function(z){
        if(length(which(prop$Mode==y & prop$Strata==z)) != 1)
          return(
            fn_keyworder(
              x = d$user_error[which(d$user_error == "UE9"), "Message"],
              lookup = data.frame(
                key = c("{Scenario_ID}"),
                value = prop$Scenario_ID[1],
                stringsAsFactors = F
              )
            )
          )
        else
          # If no problems, NULL
          NULL
      })
    })
    
    if(T %in% sapply(msg, function(y) is.character(y))){
      # Remove NULL values from msg.
      msg <- msg[sapply(msg, 
                        function(x) !is.null(x))]
      paste(
        msg,
        collapse = "<br>"
      )
    } else
      # If no problems, NULL
      NULL
  })
  
  # Throw Error message (i.e. return early with a msg).
  if(T %in% sapply(all_strata, function(y) is.character(y))){
    all_strata <- all_strata[sapply(all_strata, 
                                    function(x) !is.null(x))]
    # In other words, a fatal error has occurred.
    out$msg <- paste(
      all_strata,
      collapse = "<br>"
    )
    return(out)
  }
  
  rm(all_strata, strata, modes)
  
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  # 8. Prop Blanks 1 ----
  # Check to make sure all required Item_Result are present.
  filled <- list(
    Bus = "local",
    Car = c("arterial", "highway", "local")
  )
  
  filled_prop <- lapply(nar, function(x){
    # extract Proportion... data.frame
    prop <- x[[grep("proportion", x, ignore.case = T)[1]]]
    
    msg <- sapply(names(filled), function(y){
      sapply(filled[[y]], function(z){
        item_result <- prop$Item_Result[which(prop$Mode==y & prop$Strata==z)]
        if(
          is.nan(item_result) | 
          is.na(item_result) |
          !(is.numeric(item_result)) |
          is.infinite(item_result)
        ){

          fn_keyworder(
            x = d$user_error[which(d$user_error == "UE10"), "Message"],
            lookup = data.frame(
              key = c("{Scenario_ID}"),
              value = prop$Scenario_ID[1],
              stringsAsFactors = F
            )
          )  # end of fn_keyworder
          
        } else
          # If no problems, NULL
          NULL
      })
    })
    
    if(T %in% sapply(msg, function(y) is.character(y))){
      # Remove NULL values from msg.
      msg <- msg[sapply(msg, 
                        function(x) !is.null(x))]
      paste(
        msg,
        collapse = "<br>"
      )
    } else
      # If no problems, NULL
      NULL
  })  # end of filled_prop <- lapply
  
  # Throw Error message (i.e. return early with a msg).
  if(T %in% sapply(filled_prop, function(y) is.character(y))){
    filled_prop <- filled_prop[sapply(filled_prop, 
                                      function(x) !is.null(x))]
    # In other words, a fatal error has occurred.
    out$msg <- paste(
      filled_prop,
      collapse= "<br>"
    )
    return(out)
  }
  
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  # 9. Prop Blanks 2 ----
  # Provide warnings for those Modes/Strata that may be blank.
  # Values are substituted with Baseline 2010.
  
  may_be_blank <- list(
    Bus = c("arterial", "highway"),
    Truck = c("arterial", "highway", "local")
  )
  
  # Give warnings for: may be blank Proportions... data.frame
  mbb_prop <- lapply(nar, function(x){
    # extract Proportion... data.frame
    prop <- x[[grep("proportion", x, ignore.case = T)[1]]]
    
    msg <- sapply(names(filled), function(y){
      sapply(filled[[y]], function(z){
        item_result <- prop$Item_Result[which(prop$Mode==y & prop$Strata==z)]
        if(
          is.nan(item_result) | 
          is.na(item_result) |
          !(is.numeric(item_result)) |
          is.infinite(item_result)
        ){

          fn_keyworder(
            x = d$user_error[which(d$user_error == "UE8"), "Message"],
            lookup = data.frame(
              key = c("{Scenario_ID}", "{Mode}"),
              value = c(prop$Scenario_ID[1], y),
              stringsAsFactors = F
            )
          )  # end of fn_keyworder
          
        } else
          # If no problems, NULL
          NULL
      })
    })
    
    if(T %in% sapply(msg, function(y) is.character(y))){
      # Remove NULL values from msg.
      msg <- msg[sapply(msg, 
                        function(x) !is.null(x))]
      paste(
        msg,
        collapse = "<br>"
      )
    } else
      # If no problems, NULL
      NULL
  })  # end of mbb_prop <- lapply
  
  if(T %in% sapply(mbb_prop, function(y) is.character(y))){
    mbb_prop <- mbb_prop[sapply(mbb_prop, 
                                function(x) !is.null(x))]
    out$msg <- paste(
      out$msg,
      mbb_prop,
      collapse="<br>"
    )
    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    # Note, no return() is triggered,
    # because these blanks are just
    # Warnings.
    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    
  }
  
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  # 10. Regions Match ----
  # Check for region mismatch.
  
  # Get possible regions/geo.
  geo <- unique(d$regions$Region)
  
  # Get error messages for incorrect Regions.
  regions_match <- lapply(nar, function(x){
    
    # extract scenario_ID
    id <- x[[grep("distance", x, ignore.case = T)[1]]]$Scenario_ID[1]
    
    # extract all regions
    region <- sapply(x, function(y){
      unique(y$Region)
    })
    
    # remove duplicates
    region <- unique(unlist(region))
    
    if(length(region) != 1)
      msg <- 
      fn_keyworder(
        x = d$user_error[which(d$user_error == "UE11"), "Message"],
        lookup = data.frame(
          key = c("{Scenario_ID}"),
          value = id,
          stringsAsFactors = F
        )
      )
    else if(!(region %in% geo))
      msg <- 
      fn_keyworder(
        x = d$user_error[which(d$user_error == "UE12"), "Message"],
        lookup = data.frame(
          key = c("{Scenario_ID}"),
          value = id,
          stringsAsFactors = F
        )
      )
    else
      # If no problems, NULL
      NULL
  })
  
  # Throw Error message (i.e. return early with a msg).
  if(T %in% sapply(regions_match, function(y) is.character(y))){
    regions_match <- regions_match[sapply(regions_match, 
                                          function(x) !is.null(x))]
    # In other words, a fatal error has occurred.
    out$msg <- paste(
      regions_match,
      collapse="<br>"
    )
    return(out)
  }
  
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  # All data has been verified, so save nar back into narratives and return it.
  # Note, `out` has another element, "msg" which may or may not be NULL.
  out$continue <- T
  
  # Return
  out
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# FUNCTION: Pull Regions
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' Pull the Regions from a scenario/baseline
#' 
#' @param l is a list of 2 narratives.
#' @return vector of regions
fn_pullRegions <- function(l){
  # obtain all regions from baseline and scenario
  region1 <- unique(l[[1]]$Distances$Region)
  region2 <- unique(l[[2]]$Distances$Region)
  
  # obtain only common regions
  regions <- intersect(region1, region2)
  
  regions  # return
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# FUNCTION: Substitute User-Baseline Values
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' Substitutes values that are missing in the user's data with Baseline 2010
#' values if it falls in the range of acceptable substitutes (see 
#' `fn_user_data_verify`). By this stage, all the user data should have been 
#' verified by `fn_user_data_verify`.
#' 
#' @param nar is a list of user narratives from `fn_narrativeSplitter`.
fn_user_data_subs <- function(nar){
  
  nar <- lapply(nar, function(x){
    # user data
    dist <- x[[grep("distance", x, ignore.case = T)[1]]]
    prop <- x[[grep("proportion", x, ignore.case = T)[1]]]
    region <- dist$Region[1]
    
    # canned data
    cdist <- default_narratives$Baseline2010$Distances[
      which(default_narratives$Baseline2010$Distances$Region==region),
      ]
    
    # correct units in Distances for Baseline.
    cdist$Item_Result <- cdist$Item_Result/365
    
    cprop <- d$facility[which(d$facility$Region==region),]
    
    cprop <- reshape(
      data = cprop,
      idvar = c("Region", "Mode"),
      varying = c("local_pct_b",	"art_pct_b",	"hwy_pct_b"),
      times = c("local",	"arterial",	"highway"),
      v.name = "Item_Result",
      new.row.names = 1:(nrow(cprop) * 3),
      direction = "long"
    )
    
    # fill in missing data
    dist$Item_Result <- sapply(seq_along(dist$Item_Result), function(y){
      if(
        is.nan(dist$Item_Result[y]) | 
        is.na(dist$Item_Result[y]) |
        !(is.numeric(dist$Item_Result[y])) |
        is.infinite(dist$Item_Result[y])
      ){
        # replace with canned distance with same mode.
        cdist$Item_Result[which(cdist$Mode==dist$Mode[y])]
      } else
        # or return original value.
        dist$Item_Result[y]
    })  # end of dist$Item_Result <- sapply
    
    prop$Item_Result <- sapply(seq_along(prop$Item_Result), 
                               function(y){
      if(
        is.nan(prop$Item_Result[y]) | 
        is.na(prop$Item_Result[y]) |
        !(is.numeric(prop$Item_Result[y])) |
        is.infinite(prop$Item_Result[y])
      ){
        # replace with canned distance with same mode.
        cprop$Item_Result[which(cprop$Mode==prop$Mode[y] &
                                  cprop$time==prop$Strata[y])]
      } else
        # or return original value.
        prop$Item_Result[y]
    })  # end of prop$Item_Result <- sapply
    
    # Replace data.frames.
    x[[which(grepl("distance", names(x), ignore.case = T))[1]]] <- dist
    x[[which(grepl("proportion", names(x), ignore.case = T))[1]]] <- prop
    
    # return
    x
  })  # end of nar <- lapply
}